<?php

namespace Vtb\Pay;

use Bitrix\Main\ArgumentNullException;
use Protobuf\Exception;

error_reporting(E_ALL);

class VtbHandler
{
    private static $test = false;

    private static $logger;

    /**
     * @param $order
     * @param $params
     * @param $payment
     * @param $warrantyOrderId
     * @param $stage
     * @return false|mixed
     * @throws Exception
     */
    public function getUrl(
        $order,
        $params,
        $payment,
        $warrantyOrderId
    ) {
        self::$logger = VtbPayLogger::getInstance();

        $propertyCollection = $order->getPropertyCollection();
        $orderCreate = $order->getDateInsert()->getTimestamp();
        $amount = $order->getPrice();

        if ($propUserEmail = $propertyCollection->getUserEmail()) {
            $userEmail = $propUserEmail->getValue();
        } else {
            foreach ($propertyCollection as $orderProperty) {
                if ($orderProperty->getField('CODE') == 'EMAIL') {
                    $userEmail = $orderProperty->getValue();
                    break;
                }
            }
        }

        $email = $userEmail ?: $params['VTB_FISCAL_EMAIL'];

        $orderId = implode('-', [$warrantyOrderId, $payment->getField('ID')]); // OID + PID

        try {
            $vtbRequest = new VtbApi(
                $params['VTB_CLIENT_ID'],
                $params['VTB_CLIENT_SECRET'],
                self::$test,
                $params['VTB_MERCHANT_AUTHORIZATION'] ?: ''
            );

            $items = [];
            if ($params['VTB_STEP_STAGE'] !== 'DOUBLE' && $params['VTB_FISCAL_ENABLE'] == 'Y') $items = self::getItems($params,$order);

            $response = $vtbRequest->getOrderLink(
                $orderId,
                $email,
                $orderCreate,
                $amount,
                self::getOrderUrl($orderId, '/personal/orders/#ID#'),  // 1.3.1 - теперь адрес возврата жестко прописан
                $params['VTB_STEP_STAGE'] === 'DOUBLE' ? true : false,
                $items
            );
        } catch (\Exception | VtbPayException $e) {
            // Handle exception and log error
            $context = [
                'file_exception' => $e->getFile(),
                'line_exception' => $e->getLine(),
            ];
            if (method_exists($e, 'getContext')) $context = array_merge($e->getContext(), $context);

            self::$logger->error(sprintf(
                                __FUNCTION__ . ' > VtbPay exception : %s; Order id: %s;',
                                $e->getMessage(),
                                $orderId ?: ''
                        ), $context);

                        throw new Exception("Запрос не достиг адресата. Повторите попытку позднее.");
        }

        if (
            isset($response['object']['payUrl']) &&
            !empty($result = $response['object']['payUrl'])
        ) {
            return $result;
        }

        return false;
    }

    /**
     * Получает массив товаров заказа.
     *
     * @return array Массив товаров.
     */
    private function getItems($params,$order): array
    {

        $basket = $order->getBasket();
        $basketItems = $basket->getBasketItems();

        $items = [];
        foreach ($basketItems as $key => $item) {
            $basketPropertyCollection = $item->getPropertyCollection();
            $getPropertyValues = $basketPropertyCollection->getPropertyValues();
            $code = '';
            // 1.4.4 добавлена проверка на наличие ARTNUMBER (артикула)
            if (isset($getPropertyValues['ARTNUMBER']))
                $code = $getPropertyValues['ARTNUMBER']['VALUE'];

            $items[] = [
                'positionId' => ($key + 1),
                'name' => $item->getField('NAME'),
                'code' => $code,
                'price' => floatval($item->getPrice()),
                'measure' => (int) $params['VTB_FISCAL_MEASURE'] ?: 0,
                'quantity' => (int) $item->getQuantity(),
                'taxParams' => [
                    'taxType' => $params['VTB_FISCAL_TAX'] ?: 'none'
                ],
                'paymentType' => $params['VTB_FISCAL_PAYMENT_TYPE'] ?: 'full_prepayment',
                'paymentSubject' => (int) $params['VTB_FISCAL_SUBJECT'] ?: 1,
                'amount' => floatval($item->getPrice() * $item->getQuantity())
            ];
        }

        // Стоимость доставки
        if (!empty($order->getDeliveryPrice())) {
            $items[] = [
                'positionId' => ($key + 2),
                'name' => 'Доставка',
                'price' => floatval($order->getDeliveryPrice()),
                'measure' => 0,
                'quantity' => 1,
                'taxParams' => [
                    'taxType' => $params['VTB_FISCAL_TAX'] ?: 'none'
                ],
                'paymentType' => $params['VTB_FISCAL_DELIVERY'] ?: 'full_prepayment',
                'paymentSubject' => 4,
                'amount' => floatval($order->getDeliveryPrice())
            ];
        }

        return $items;
    }


    /**
     * @param string $orderId
     * @return string
     */
    private static function getOrderUrl(string $orderId, string $returnUrl): string
    {
        $protocol = (empty($_SERVER['HTTPS']) ? 'http' : 'https');
        $host = $_SERVER['HTTP_HOST'];
        $orderPage = str_replace('#ID#', $orderId, $returnUrl);

        $orderUrl = "{$protocol}://{$host}{$orderPage}";

        return $orderUrl;
    }


    /**
     * @param $order
     * @param $clientId
     * @param $clientSecret
     * @param $clientMerchant
     * @param $warrantyOrderId
     * @return array|false
     * @throws Exception
     */
    public static function getOrderInfo(
        $order,
        $paymentId,
        $params,
        $warrantyOrderId
    )
    {
        $result = false;

        self::$logger = VtbPayLogger::getInstance();

        $orderId = implode('-', [$warrantyOrderId, $paymentId]); // OID + PID

        try {
            $vtbRequest = new VtbApi(
                $params['VTB_CLIENT_ID'],
                $params['VTB_CLIENT_SECRET'],
                self::$test,
                $params['VTB_MERCHANT_AUTHORIZATION'] ?: ''
            );
            $response = $vtbRequest->getOrderInfo($orderId);
        } catch (\Exception | VtbPayException $e) {
            // Handle exception and log error
            $context = [
                'file_exception' => $e->getFile(),
                'line_exception' => $e->getLine(),
            ];
            if (method_exists($e, 'getContext')) $context = array_merge($e->getContext(), $context);

            self::$logger->error(sprintf(
                                __FUNCTION__ . ' > VtbPay exception : %s; Order id: %s;',
                                $e->getMessage(),
                                $orderId ?: ''
                        ), $context);

                        throw new Exception("Запрос не достиг адресата. Повторите попытку позднее.");
        }

        if (is_array($response)) {
            $result = $response;
        }

        return $result;
    }

    /**
     * @param $orderId
     * @return mixed
     */
    public static function setPaymentRefunds($params, $orderId, $paymentId)
    {
        self::$logger = VtbPayLogger::getInstance();

        try {
            $finalOrderId = implode('-', [$orderId, $paymentId]);

            $vtbRequest = new VtbApi(
                $params['VTB_CLIENT_ID'],
                $params['VTB_CLIENT_SECRET'],
                self::$test,
                $params['VTB_MERCHANT_AUTHORIZATION'] ?: ''
            );
            $refund = $vtbRequest->setRefunds($finalOrderId);

        } catch (\Exception | VtbPayException $e) {
            // Handle exception and log error
            $context = [
                'file_exception' => $e->getFile(),
                'line_exception' => $e->getLine(),
            ];
            if (method_exists($e, 'getContext')) $context = array_merge($e->getContext(), $context);

            self::$logger->error(sprintf(
                                __FUNCTION__ . ' > VtbPay exception : %s; Order id: %s;',
                                $e->getMessage(),
                                $orderId ?: ''
                        ), $context);

            throw $e;
        }

        return $refund;
    }

    /**
     * @param bool $test
     * @return void
     */
    public static function setTest(bool $test = true)
    {
        self::$test = $test;
    }
}
