<?php

use Bitrix\Main\Application;
use Bitrix\Main\Localization\Loc;

IncludeModuleLangFile(__FILE__);

class vtbpayments_ecom extends CModule
{
    var $MODULE_ID = 'vtbpayments.ecom';
    var $MODULE_SALE = 'vtbpayments';
    var $MODULE_VERSION;
    var $MODULE_VERSION_DATE;
    var $MODULE_GROUP_RIGHTS = 'N';
    var $MODULE_DEBUG = 'Y';
    var $MODULE_STATUSES;

    function __construct()
    {
        $arModuleVersion = [];

        // If BX_ROOT is not defined
        if (!defined('BX_ROOT')) {
            if (file_exists($_SERVER['DOCUMENT_ROOT'] . '/bitrix/modules/main/bx_root.php')) {
                require '/bitrix/modules/main/bx_root.php';
            } else {
                define('BX_ROOT', '/bitrix');
            }
        }

        include __DIR__ . '/version.php';

        if (is_array($arModuleVersion) && array_key_exists('VERSION', $arModuleVersion)) {
            $this->MODULE_VERSION = $arModuleVersion['VERSION'];
            $this->MODULE_VERSION_DATE = $arModuleVersion['VERSION_DATE'];
        }

        $this->MODULE_NAME = Loc::getMessage('VTB_MODULE_NAME');
        $this->MODULE_DESCRIPTION = Loc::getMessage('VTB_MODULE_DESCRIPTION');

        $this->PARTNER_NAME = Loc::getMessage('VTB_MODULE_AUTHOR');
        $this->PARTNER_URI = Loc::getMessage('VTB_MODULE_LINK');

        $this->MODULE_STATUSES = [
            [
                'ID' => 'VP', // VTB Pending status
                'SORT' => 100,
                'LANG' => [
                    [
                        'LID' => 'en',
                        'NAME' => 'VTB: Pending'
                    ], [
                        'LID' => 'ru',
                        'NAME' => Loc::getMessage('VTB_ORDER_STATUS_VP')
                    ]
                ]
            ], [
                'ID' => 'VR', // VTB Refund
                'SORT' => 101,
                'LANG' => [
                    [
                        'LID' => 'en',
                        'NAME' => 'VTB: Refund'
                    ], [
                        'LID' => 'ru',
                        'NAME' => Loc::getMessage('VTB_ORDER_STATUS_VR')
                    ]
                ]
            ], [
                'ID' => 'VC', // VTB Refund
                'SORT' => 102,
                'LANG' => [
                    [
                        'LID' => 'en',
                        'NAME' => 'VTB: Cancel'
                    ], [
                        'LID' => 'ru',
                        'NAME' => Loc::getMessage('VTB_ORDER_STATUS_VC')
                    ]
                ]
            ]
        ];
    }

    function DoInstall()
    {
        global $APPLICATION;

        if (!IsModuleInstalled($this->MODULE_ID)) {
            RegisterModule($this->MODULE_ID);

            $this->InstallFiles();

            $this->InstallOrderStatus();

            $APPLICATION->ThrowException(Loc::getMessage("VTB_SUCCESS_INSTALL"));

            $this->vtbLoggerInstall('Module register', 'DoInstall');
        } else {
            $APPLICATION->ThrowException(Loc::getMessage("VTB_ERROR_INSTALL"));

            $this->vtbLoggerInstall('Module do not register', 'DoInstall');
        }
    }

    function InstallFiles($arParams = [])
    {
        // HANDLER MOVED
        $psLinkInstallComplete = CopyDirFiles(__DIR__ . BX_ROOT, Application::getDocumentRoot() . BX_ROOT, true, true);
        $this->vtbLoggerInstall('Files copying: ' . $psLinkInstallComplete, 'InstallFiles()');
        if (!$psLinkInstallComplete) {
            echo \CAdminMessage::ShowMessage(Loc::getMessage('VTB_MODULE_INSTALL_HANDLER_ERROR'));
        }

        CopyDirFiles($_SERVER["DOCUMENT_ROOT"]."/bitrix/modules/vtbpayments.ecom/install/images", $_SERVER["DOCUMENT_ROOT"]."/bitrix/images/sale/sale_payments"); // 1.4.6 - установка логотипа втб при установке модуля

        // MEDIA MOVED
        $psLinkMediaComplete = CopyDirFiles(__DIR__ . "/install/images", $this->MODULE_MEDIA_PATH, true, true);
        $this->vtbLoggerInstall('Media copying: ' . $psLinkMediaComplete, 'InstallFiles()');
        if (!$psLinkMediaComplete) {
            echo \CAdminMessage::ShowMessage(Loc::getMessage('VTB_MODULE_INSTALL_MEDIA_ERROR'));
        }

        $this->vtbLoggerInstall('Method finish', 'InstallFiles');
    }

    function InstallOrderStatus()
    {
        if (!CModule::IncludeModule("sale")) {
            $this->vtbLoggerInstall('Sale module not included', 'InstallOrderStatus');
            return false;
        }

        foreach ($this->MODULE_STATUSES as $status) {
            CSaleStatus::Add($status);
            $this->vtbLoggerInstall($status, 'InstallOrderStatus');
        }
    }

    function DoUninstall()
    {
        UnRegisterModule($this->MODULE_ID);

        $this->UnInstallFiles();

        $this->UnInstallStatuses();

        $this->UnInstallBussVal();

        $this->vtbLoggerInstall('Module unregister', 'DoUninstall');

        return true;
    }

    function UnInstallFiles()
    {
        // Handlers
        DeleteDirFilesEx($_SERVER['DOCUMENT_ROOT'] . BX_ROOT . '/' . $this->MODULE_SALE . '/');
        // Logotype
        DeleteDirFilesEx($_SERVER['DOCUMENT_ROOT'] . BX_ROOT . '/' . $this->MODULE_SALE . '.png');

        return true;
    }

    function UnInstallStatuses()
    {
        if (!CModule::IncludeModule("sale")) {
            return false;
        }

        foreach ($this->MODULE_STATUSES as $status) {
            CSaleStatus::Delete($status['ID']);
            $this->vtbLoggerInstall($status, 'UnInstallStatuses');
        }
    }

    function UnInstallBussVal()
    {
        $optionList = [
            'VTB_CLIENT_ID',
            'VTB_CLIENT_SECRET',
            'VTB_MERCHANT_AUTHORIZATION',
            'VTB_TEST_MODE',
            'VTB_ORDER_STATUS',
            'VTB_STEP_STAGE',
            'VTB_FISCAL_ENABLE', // 1.4.3 - Добавлена настройка вкл/выкл фискализации
            'VTB_FISCAL_EMAIL',
            'VTB_FISCAL_DELIVERY',
            'VTB_FISCAL_MEASURE',
            'VTB_FISCAL_TAX',
            'VTB_FISCAL_PAYMENT_TYPE',
            'VTB_FISCAL_SUBJECT',
        ];
        foreach ($optionList as $option) {
            COption::RemoveOption('vtbpayments.ecom', $option);
        }

        $this->vtbLoggerInstall('BusinessValue removed', 'UnInstallBussVal');
    }

    private function vtbLoggerInstall($data = null, $method = null)
    {
        $date = date('[Y-m-d H:i:s]');
        $string = "{$date} ";

        if (!empty($method)) {
            $string .= "{$method} ";
        }

        if (is_string($data)) {
            $string .= $data;
        } else if (is_array($data) || is_object($data)) {
            $string .= json_encode($data, JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES);
        } else {
            $string .= 'data is empty';
        }

        if (!file_exists($_SERVER['DOCUMENT_ROOT'] . BX_ROOT . "/modules/vtbpayments.ecom/logs")) {
            mkdir($_SERVER['DOCUMENT_ROOT'] . BX_ROOT . "/modules/vtbpayments.ecom/logs", 0755, true);
        }

        $logFile = $_SERVER['DOCUMENT_ROOT'] . BX_ROOT . "/modules/vtbpayments.ecom/logs/install.log";
        file_put_contents($logFile, $string . PHP_EOL, FILE_APPEND);
    }
}